require 'fileutils'
require 'pathname'
require 'fakefs/base'
require 'fakefs/fake/file'
require 'fakefs/fake/dir'
require 'fakefs/fake/symlink'
require 'fakefs/file_system'
require 'fakefs/fileutils'
require 'fakefs/file'
require 'fakefs/file_test'
require 'fakefs/dir'
require 'fakefs/pathname' if RUBY_VERSION >= "1.9.3"

